from bisect import *
from pylab import *

set_printoptions(3)

def f_pendulum(t, u):
    E = 10.0
    D = 4.0
    L = 1.0
    m = 1.0

    x = u[0:2]
    v = u[2:4]

    x0 = array([0.0, 0.0])
    v0 = array([0.0, 0.0])

    i = 0

    F01 = 0.0
    r01 = norm(x - x0)
    e01 = (x - x0) / r01
    F01 = -E*(r01 - L)*e01 - D*dot(v - v0, e01)*e01

    g = array([0.0, -1.0*0.1])

    val = zeros(u.size)
    val[0:2] = v
    val[2:4] = F01 / m + g

    return val

def piecewise_linear_adapter(tarray, uarray):
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[:, i0]
        u1 = uarray[:, i1]

        # Lagrange polynomial weights
        w0 = (t1 - t) / (t1 - t0)
        w1 = (t - t0) / (t1 - t0)

        ut = w0*u0 + w1*u1
        return ut

    return u_adapted

def derivative_adapter(tarray, uarray):
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[:, i0]
        u1 = uarray[:, i1]

        k = t1 - t0
        dudt = (u1 - u0) / (t1 - t0)

        ut = dudt
        return ut

    return u_adapted

def jacobian(f, t, u):

    y = f(t, u)
    if(size(u) > 1):
        J = zeros([size(y), size(u)])
    else:
        J = 0.0

    h = 1.0e-7
    for i in range(0, size(u)):
        hv = zeros([size(u)])
        hv[i] = h
        Jcol = (f(t, u + hv) - f(t, u - hv)) / (2*h)

        if(size(u) > 1):
            J[:, i] = Jcol
        else:
            J = Jcol
        hv[i] = 0.0

    return J

def linearization(f, ulin):
    def flin(t, u):
        J = jacobian(f, t, ulin(t))
        return dot(J, u)

    return flin

def f3body(t, u):
    # Fill in yourself
    return 0

def Sd_estimate(f, I):
    # Fill in yourself
    return 0

def Sc_estimate(f, I):
    # Fill in yourself
    return 0
